@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
-# $Revision: 1.24 $ 
-# $Date: 2005/05/11 20:19:27 $
+# $Revision: 1.30 $ 
+# $Date: 2005/07/18 20:28:20 $
 # $Author: dwelch $
 #
 # (c) Copyright 2003-2004 Hewlett-Packard Development Company, L.P.
@@ -74,6 +74,8 @@ ERROR_SMTP_SENDER_REFUSED = 43
 ERROR_SMTP_DATA_ERROR = 44
 ERROR_INVALID_HOSTNAME = 45
 ERROR_INVALID_PORT_NUMBER = 46
+ERROR_NO_CUPS_QUEUE_FOUND_FOR_DEVICE = 47
+ERROR_UNSUPPORTED_MODEL = 48
 ERROR_UNABLE_TO_BIND_SOCKET = 95
 # --> add new codes here <--
 ERROR_UNABLE_TO_CONTACT_SERVICE = 96
@@ -178,6 +180,11 @@ STATUS_PRINTER_OFFLINE = 1803
 STATUS_PRINTER_LOW_TONER = 1804
 STATUS_PRINTER_NO_TONER = 1805
 STATUS_PRINTER_SERVICE_REQUEST = 1806
+#end
+
+# other "synthetic" codes
+STATUS_DEVICE_UNSUPPORTED = 1900
+#end
 
 # scan
 EVENT_START_SCAN_JOB = 2000
@@ -327,6 +334,7 @@ STATUS_TO_ERROR_STATE_MAP = {
     STATUS_PRINTER_LOW_TONER : ERROR_STATE_LOW_SUPPLIES,
     STATUS_PRINTER_NO_TONER : ERROR_STATE_LOW_SUPPLIES,
     STATUS_PRINTER_SERVICE_REQUEST : ERROR_STATE_ERROR,
+    STATUS_DEVICE_UNSUPPORTED : ERROR_STATE_ERROR,
     # The following block are EVENTs because they are only
     # recieved as events from hpiod, hp backend, etc.
     # i.e., a device does not produce status codes in this range
@@ -487,6 +495,7 @@ COLOR_CAL_TYPE_NONE = 0
 COLOR_CAL_TYPE_DESKJET_450 = 1
 COLOR_CAL_TYPE_MALIBU_CRICK = 2
 COLOR_CAL_TYPE_STRINGRAY_LONGBOW_TORNADO = 3
+COLOR_CAL_TYPE_CONNERY = 4
 
 # status-types
 STATUS_TYPE_NONE = 0
@@ -553,3 +562,25 @@ IN_TRAY_NOT_PRESENT = 0
 IN_TRAY_PRESENT = 1 # for !S:02, test for > IN_TRAY_NOT_PRESENT
 IN_TRAY_DEFAULT = 2 # S:02 only
 IN_TRAY_LOCKED = 3 # S:02 only
+
+# 'io-support'
+IO_SUPPORT_PARALLEL = 0x1
+IO_SUPPORT_USB = 0x2
+IO_SUPPORT_NETWORK = 0x4
+IO_SUPPORT_WIRELESS = 0x8
+IO_SUPPORT_BLUETOOTH = 0x10
+
+# Model categories
+MODEL_TYPE_COLOR_INKJET = 0
+MODEL_TYPE_PHOTOSMART = 1
+MODEL_TYPE_COLOR_LASER = 2
+MODEL_TYPE_COLOR_LASER_AIO = 3
+MODEL_TYPE_COLOR_INKJET_AIO = 4
+MODEL_TYPE_MONO_LASER = 5
+MODEL_TYPE_MONO_LASER_AIO = 6
+
+# 'io-mode'
+IO_MODE_UNI = 0
+IO_MODE_RAW = 1
+IO_MODE_MLC = 2
+IO_MODE_DOT4 = 3
